﻿//The code is written by Andrew Polar, the concept is designed tother with Mike Poluektov
//It is simple DEMO of mushroom classification
//Dataset: https://archive.ics.uci.edu/ml/datasets/mushroom

using System;
using System.Linq;

namespace M_class
{
    class Program
    {
        static int IdentificationAndValidation(DataHolder dh, bool[] isValidation, int fold)
        {
            int steps = 100;
            double mu = 0.5;
            UQuant uq = new UQuant();

            //Identification
            for (int step = 0; step < steps; ++step)
            {
                for (int i = 0; i < dh._target.Count; ++i)
                {
                    if (true == isValidation[i]) continue;
                    double target = dh._target[i];
                    int[] address = dh._input[i];
                    double output = 0.0;
                    for (int j = 0; j < address.Length; ++j)
                    {
                        output += uq._U[j][address[j] - 1];
                    }
                    double diff = target - output;
                    diff /= address.Length;
                    diff *= mu;
                    for (int j = 0; j < address.Length; ++j)
                    {
                        uq._U[j][address[j] - 1] += diff;
                    }
                }
            }

            //Validation
            int right = 0;
            int total = 0;
            for (int i = 0; i < dh._target.Count; ++i)
            {
                if (!isValidation[i]) continue;

                double target = dh._target[i];
                int[] address = dh._input[i];
                double output = 0.0;
                for (int j = 0; j < address.Length; ++j)
                {
                    output += uq._U[j][address[j] - 1];
                }
                if (target > 0.0 && output > 0.0) ++right;
                if (target <= 0.0 && output <= 0.0) ++right;
                ++total;
            }
            Console.WriteLine();
            Console.WriteLine("Training and validation session {0}, result for validation block:", fold);
            Console.WriteLine("total = {0}, correct = {1}, errors = {2}", total, right, total - right);

            return total - right;
        }


        static void Main(string[] args)
        {
            DataHolder dh = new DataHolder();
            dh.ProcessDataFile(@"..\..\mushrooms.csv");

            DateTime start = DateTime.Now;

            int NFolds = 10;
            int records = dh._target.Count();
            int blockSize = records / NFolds;
            bool[] isValidation = new bool[records];

            ////////
            int totalErrors = 0;
            for (int fold = 0; fold < NFolds; ++fold)
            {
                for (int i = 0; i < isValidation.Length; ++i)
                {
                    isValidation[i] = false;
                }

                for (int i = blockSize * fold; i < blockSize * (fold + 1); ++i)
                {
                    isValidation[i] = true;
                }
                totalErrors += IdentificationAndValidation(dh, isValidation, fold + 1);
            }

            Console.WriteLine("\n------------------------------------------");
            Console.WriteLine("Total errors for 10 fold validation {0}", totalErrors);

            DateTime end = DateTime.Now;
            TimeSpan duration = end - start;
            double time = duration.Minutes * 60.0 + duration.Seconds + duration.Milliseconds / 1000.0;
            Console.WriteLine("Total time for {0} training sessions {1:####.00} seconds", NFolds, time);
        }
    }
}
